import { render } from 'react-dom';
import { createElement } from 'react';

// A type of promise-like that resolves synchronously and supports only one observer

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

var componentSelector = 'data-component';
var components = {};
var unPopulatedElements = [];
var register = function register(name, fn) {
  components[name] = retry(fn, 10, 20);
};
var delay = function delay(ms) {
  return new Promise(function (res) {
    return setTimeout(res, ms);
  });
};

var retry = function retry(fn, times, delayTime) {
  try {
    return Promise.resolve(_catch(fn, function (err) {
      if (times > 1) {
        return Promise.resolve(delay(delayTime)).then(function () {
          return retry(fn, times - 1, delayTime * 2);
        });
      } else {
        throw new Error(err);
      }
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};

var setComponentSelector = function setComponentSelector(selector) {
  componentSelector = selector;
};
var getRegisteredComponents = function getRegisteredComponents() {
  return components;
};
var getActiveComponents = function getActiveComponents() {
  return Array.from(new Set(getAbodeElements().map(function (el) {
    return el.getAttribute(componentSelector);
  })));
}; // start prop logic

var getCleanPropName = function getCleanPropName(raw) {
  return raw.replace('data-prop-', '').replace(/-./g, function (x) {
    return x.toUpperCase()[1];
  });
};
var getElementProps = function getElementProps(el) {
  var props = {};

  if (el !== null && el !== void 0 && el.attributes) {
    var rawProps = Array.from(el.attributes).filter(function (attribute) {
      return attribute.name.startsWith('data-prop-');
    });
    rawProps.forEach(function (prop) {
      return props[getCleanPropName(prop.name)] = prop.value;
    });
  }

  return props;
};
var getScriptProps = function getScriptProps() {
  var element = document.currentScript;
  return getElementProps(element);
}; // end prop logic
// start element logic

var getAbodeElements = function getAbodeElements() {
  return Array.from(document.querySelectorAll("[" + componentSelector + "]")).filter(function (el) {
    var component = el.getAttribute(componentSelector); // It should exist in registered components

    return component && components[component];
  });
};
var setUnpopulatedElements = function setUnpopulatedElements() {
  unPopulatedElements = getAbodeElements().filter(function (el) {
    return !el.getAttribute('react-abode-populated');
  });
};
var setAttributes = function setAttributes(el, attributes) {
  Object.entries(attributes).forEach(function (_ref) {
    var k = _ref[0],
        v = _ref[1];
    return el.setAttribute(k, v);
  });
}; // end element logic

var renderAbode = function renderAbode(el) {
  try {
    var _Array$from$find;

    var props = getElementProps(el);
    var componentName = (_Array$from$find = Array.from(el.attributes).find(function (at) {
      return at.name === componentSelector;
    })) === null || _Array$from$find === void 0 ? void 0 : _Array$from$find.value;

    if (!componentName || componentName === '') {
      throw new Error("not all react-abode elements have a value for  " + componentSelector);
    }

    return Promise.resolve(components[componentName]).then(function (module) {
      if (!module) {
        throw new Error("no component registered for " + componentName);
      }

      var element = module["default"] || module;
      render(createElement(element, props), el);
    });
  } catch (e) {
    return Promise.reject(e);
  }
};
var trackPropChanges = function trackPropChanges(el) {
  if (MutationObserver) {
    var observer = new MutationObserver(function () {
      renderAbode(el);
    });
    observer.observe(el, {
      attributes: true
    });
  }
};
var update = function update(elements, options) {
  try {
    // tag first, since adding components is a slow process and will cause components to get iterated multiple times
    elements.forEach(function (el) {
      return el.setAttribute('react-abode-populated', 'true');
    });
    elements.forEach(function (el) {
      if (options !== null && options !== void 0 && options.attributes) setAttributes(el, options.attributes);
      renderAbode(el);
      trackPropChanges(el);
    });
    return Promise.resolve();
  } catch (e) {
    return Promise.reject(e);
  }
};

var checkForAndHandleNewComponents = function checkForAndHandleNewComponents(options) {
  try {
    setUnpopulatedElements();

    var _temp2 = function () {
      if (unPopulatedElements.length) {
        return Promise.resolve(update(unPopulatedElements, options)).then(function () {
          unPopulatedElements = [];
          if (options !== null && options !== void 0 && options.callback) options.callback();
        });
      }
    }();

    return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(function () {}) : void 0);
  } catch (e) {
    return Promise.reject(e);
  }
};

var populate = function populate(options) {
  try {
    return Promise.resolve(checkForAndHandleNewComponents(options)).then(function () {
      document.body.addEventListener('DOMNodeInserted', function () {
        return checkForAndHandleNewComponents(options);
      });
    });
  } catch (e) {
    return Promise.reject(e);
  }
};

export { getActiveComponents, getRegisteredComponents, getScriptProps, populate, register, setComponentSelector };
//# sourceMappingURL=react-abode.esm.js.map
